/*
 * Copyright (c) 2019 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: LicenseRef-BSD-5-Clause-Nordic
 */

#include <event_manager.h>
#include <METCO_uart.h>
#include <led.h>
#include <button.h>
#include <gps.h>


#include <logging/log.h>

//needed for nvs
#include <flash.h>
#include <nvs/nvs.h>


LOG_MODULE_REGISTER(MODULE);

struct nvs_fs fs;


//--------------------------------


int nvs_setup(){
  int retval = 0;
  struct flash_pages_info info;

  fs.offset = DT_FLASH_AREA_STORAGE_OFFSET;
  retval = flash_get_page_info_by_offs(device_get_binding(DT_FLASH_DEV_NAME), fs.offset, &info);
  if(retval){
    printk("Unable to get page info");
    return retval;
  }

  fs.sector_size = info.size;
  fs.sector_count = 3U;

  retval = nvs_init(&fs, DT_FLASH_DEV_NAME);
  if(retval){
    printk("Flash Init failed\n");
    return retval;
  }
  return 0;
}

void main(void)
{

        uart_init();
        printk("UARTs Ready\n");

        nvs_setup();
        printk("NVS Setup\n");
        
        led_setup();
        printk("LEDs Ready\n");

        button_setup();
        printk("Buttons Ready\n");   
            
        GPS_Init();
        printk("\nGPS Initialized");

        GPS_Enable();

	if (event_manager_init()) {
		LOG_ERR("Event Manager not initialized");
	} else {

		printk("Event Manager Initialized");
	}

        s64_t loop_timer = k_uptime_get();

        while(1)
        {
            GPS_State_Machine(&fs);

            while(k_uptime_get()-loop_timer <= 20);
            loop_timer = k_uptime_get();
            //k_cpu_idle();
        }
}
