
#include <zephyr.h>
#include <device.h>
#include <gpio.h>
#include <led.h>

struct device *dev0;
struct device *dev1;
struct device *dev2;
struct device *dev3;

void led_setup(void){
    dev0 = device_get_binding(LED0_PORT);
    dev1 = device_get_binding(LED1_PORT);
    dev2 = device_get_binding(LED2_PORT);
    dev3 = device_get_binding(LED3_PORT);

    gpio_pin_configure(dev0, LED0, GPIO_DIR_OUT);
    gpio_pin_configure(dev1, LED1, GPIO_DIR_OUT);
    gpio_pin_configure(dev2, LED2, GPIO_DIR_OUT);
    gpio_pin_configure(dev3, LED3, GPIO_DIR_OUT);
}

struct device* get_led_device(int led_num){
    switch(led_num){
        case 1:
          return dev0;
        case 2:
          return dev1;
        case 3:
          return dev2;
        case 4:
          return dev3;
        default:
          return dev0;
    }
}

u32_t get_led_pin(int led_num){
    switch(led_num){
        case 1:
          return LED0_GPIO_PIN;
        case 2:
          return LED1_GPIO_PIN;
        case 3:
          return LED2_GPIO_PIN;
        case 4:
          return LED3_GPIO_PIN;
        default:
          return LED0_GPIO_PIN;
    }
}

u32_t get_led_status(int led_num){
    u32_t on_off;
    struct device *dev = get_led_device(led_num);
    u32_t pin = get_led_pin(led_num);

    gpio_pin_read(dev, pin, &on_off);
    return on_off;
}

void toggle_led(int led_num){
    struct device *dev = get_led_device(led_num);
    u32_t pin = get_led_pin(led_num);
    u32_t status = get_led_status(led_num);
    gpio_pin_write(dev, pin, !status);

}

void led_on(int led_num){
    struct device *dev = get_led_device(led_num);
    u32_t pin = get_led_pin(led_num);
    gpio_pin_write(dev, pin, 1);
}

void led_off(int led_num){
    struct device *dev = get_led_device(led_num);
    u32_t pin = get_led_pin(led_num);
    gpio_pin_write(dev, pin, 0);
}
