/*

* Copyright (c) 2012-2014 Wind River Systems, Inc.

*

* SPDX-License-Identifier: Apache-2.0

*/

#include <zephyr.h>
#include <misc/printk.h>
#include <uart.h>
#include <string.h>
#include <stdlib.h>
#include <METCO_uart.h>
#include <uart_event.h>

#define MODULE uart

struct uart_buffer *ptr0, buffer0;
struct uart_buffer *ptr1, buffer1;

void clear_uart_buf(struct uart_buffer *p);
void fill_buffer(struct device *x, int length,  struct uart_buffer *p);
void uart_send_str(struct device *uart, char *str);
void uart_cb0(struct device *x);
void uart_cb1(struct device *x);


//initializes the UARTs
void uart_init()
{
        //initialize uart0
    	struct device *uart0 = device_get_binding("UART_0");
	uart_irq_callback_set(uart0, uart_cb0);
	uart_irq_rx_enable(uart0);
        buffer0.buf_loc_id = 0;
        buffer0.cleared = true;
        ptr0 = &buffer0;

        //initialize uart1
        struct device *uart1 = device_get_binding("UART_1");
	uart_irq_callback_set(uart1, uart_cb1);
	uart_irq_rx_enable(uart1);
        buffer1.buf_loc_id = 0;
        buffer1.cleared = true;
        ptr1 = &buffer1;
}

//callback for UART0 when data arrives
void uart_cb0(struct device *x) 
{
        
	uart_irq_update(x); //start processing interrupts in the ISR
	int data_length = 0;
        
	if (uart_irq_rx_ready(x)) { //check if UART RX buffer has a received char
		data_length = uart_fifo_read(x, ptr0->uart_buf, sizeof(ptr0->uart_buf)); //read data from FIFO, used pointers instead of passing full array
		buffer0.uart_buf[data_length] = 0;
		fill_buffer(x, data_length, ptr0);
	}

}

//callback for UART0 when data arrives
void uart_cb1(struct device *x) //callback for UART1 when data arrives
{
	uart_irq_update(x); //start processing interrupts in the ISR
	int data_length = 0;

	if (uart_irq_rx_ready(x)) {  //check if UART RX buffer has a received char
		data_length = uart_fifo_read(x, ptr1->uart_buf, sizeof(ptr1->uart_buf));  //read data from FIFO, used pointers instead of passing full array
		buffer1.uart_buf[data_length] = 0;
		fill_buffer(x, data_length, ptr1);
	}       

}

//Fills the permenant buffer 'buf' from the temporary buffer 'uart_buffer'
void fill_buffer(struct device *x, int length,  struct uart_buffer *p)
{
        int i=0;
        //Puts each byte of data in its respective buffer location 
        //buff_loc_id is used to identify where in the buffer to place each byte
        for (i=0; i < length; i++, p->buf_loc_id++)  
        {
            p->buf[p->buf_loc_id] = p->uart_buf[i];
        }

        //ASCI '1' currently signifies an end of message byte and will trigger a message event to be processed
        if (p->buf[p->buf_loc_id-1] == '1')
        {
            //Allocate event
            struct uart_event_test *event = new_uart_event_test();  

            //write data to event datafield
            event->ptr_to_buffer = p;
            event->ptr_to_uart_device = x;

            //submit event
            EVENT_SUBMIT(event);

        }
        else
        {
            p->cleared = false;
        }

}

//clear the permenant buffer 'buf' for the respective UART
void clear_uart_buf(struct uart_buffer *p)
{
        memset(p->buf, 0, sizeof(p->buf)); //set everything in the buffer to zeros
        p->buf_loc_id = 0; //reset location ID within buffer to 0
        p->cleared = true;  //setting the flag to indicate that the permenant buffer has been emptied
}


void uart_send_str(struct device *uart, char *str)
{
    u32_t len = strlen(str);
    while (len--) {
        uart_poll_out(uart, *str++);
    }

}

//Event handler for UART events
static bool comm_event_handler(const struct event_header *eh)
{
        if (is_uart_event_test(eh))
        {
            //access event data
            struct uart_event_test *event = cast_uart_event_test(eh);

            //printout data and clear buff
            uart_send_str(event->ptr_to_uart_device, event->ptr_to_buffer->buf);
            clear_uart_buf(event->ptr_to_buffer);

            return false;

        }

        return false;
}

EVENT_LISTENER(MODULE, comm_event_handler);
EVENT_SUBSCRIBE(MODULE, uart_event_test);


